@@ -1,15 +1,14 @@
-import functools
 from typing import Union, NewType, Optional, List, Dict, Final
 
+import ipywidgets as widgets
 import k3d
 import numpy as np
-from k3d.objects import Group, Line, Points, Text, Drawable
+from k3d.objects import Group, Line, Points, TextureText, Drawable
 from k3d.plot import Plot
 
 from .jointset import JointSet, MuPoTSJoints, OpenPoseJoints, CocoExJoints, PanopticJoints, Common14Joints
 
 SkeletonPlot = NewType('SkeletonPlot', Optional[Plot])
-SkeletonObject = NewType('SkeletonObject', Optional[Group])
 Color = NewType('Color', Union[str, int])
 
 DEFAULT_COLORS: Final[Dict[str, int]] = {
@@ -19,6 +18,23 @@ DEFAULT_COLORS: Final[Dict[str, int]] = {
 }
 
 
+class DrawableSkeleton(Group):
+    def __init__(self, joint_points: Points, joint_lines: List[Line], joint_names: List[TextureText]):
+        drawable_objects: List[Drawable] = [joint_points] + joint_lines + joint_names
+        super().__init__(drawable_objects)
+        self.joint_points: Points = joint_points
+        self.joint_lines: List[Line] = joint_lines
+        self.joint_names: List[TextureText] = joint_names
+
+    def hide_joint_names(self):
+        for joint_name in self.joint_names:
+            joint_name.visible = False
+
+    def show_joint_names(self):
+        for joint_name in self.joint_names:
+            joint_name.visible = True
+
+
 class Skeleton:
     def __init__(self, joint_coordinates: np.ndarray,
                  joint_set: JointSet, part_size: float, color: Color):
@@ -27,17 +43,11 @@ class Skeleton:
         self.part_size: float = part_size
         self.color: Color = color
 
-    def to_skeleton_object(self) -> SkeletonObject:
+    def to_drawable_skeleton(self) -> DrawableSkeleton:
         joint_points = self.get_joint_points()
-        joint_lines: List[Drawable] = self.get_joint_lines()
-        joint_names: List[Drawable] = self.get_joint_names()
-        skeleton_object: SkeletonObject = SkeletonObject(Group())
-        skeleton_object += joint_points
-        return functools.reduce(
-            lambda _skeleton_object, _skeleton_part: _skeleton_object + _skeleton_part,
-            joint_lines + joint_names,
-            skeleton_object
-        )
+        joint_lines: List[Line] = self.get_joint_lines()
+        joint_names: List[TextureText] = self.get_joint_names()
+        return DrawableSkeleton(joint_points, joint_lines, joint_names)
 
     def get_joint_colors(self) -> np.ndarray:
         joint_colors_shape = (self.joint_set.number_of_joints, )
@@ -80,11 +90,11 @@ class Skeleton:
             for i in range(len(self.joint_set.limb_graph))
         ]
 
-    def get_joint_names(self) -> List[Text]:
-        return [k3d.text(
+    def get_joint_names(self) -> List[TextureText]:
+        return [k3d.texture_text(
             text=self.joint_set.names[i],
             position=self.joint_coordinates[i],
-            size=0.6, label_box=False)
+            size=0.45, color=DEFAULT_COLORS.get('black'))
             for i in range(self.joint_set.number_of_joints)
         ]
 
@@ -101,6 +111,7 @@ class SkeletonVisualizer:
         self.joint_set: JointSet = joint_set
         self.plot: SkeletonPlot = None
         self.size_scalar: float = size_scalar
+        self.skeletons: List[DrawableSkeleton] = []
 
     def visualize(self, skeletons: np.ndarray, colors: List[Color] = None):
         self.__assert_visualization_arguments(skeletons, colors)
@@ -108,6 +119,8 @@ class SkeletonVisualizer:
         if colors is None:
             colors = ['default' for i in range(number_of_skeletons)]
         self.plot = self.__create_skeleton_plot(skeletons, colors)
+        for skeleton in self.skeletons:
+            skeleton.hide_joint_names()
         self.plot.display()
 
     def __assert_visualization_arguments(self, skeletons: np.ndarray, colors: List[Color]):
@@ -129,7 +142,9 @@ class SkeletonVisualizer:
                     part_size=skeleton_part_size,
                     color=skeleton_color_tuple[1]),
                 zip(skeletons, colors)):
-            skeleton_plot += skeleton.to_skeleton_object()
+            drawable_skeleton = skeleton.to_drawable_skeleton()
+            skeleton_plot += drawable_skeleton
+            self.skeletons.append(drawable_skeleton)
         return skeleton_plot
 
     def __calculate_skeleton_part_size(self, skeletons: np.ndarray) -> float:
